#include <qdrawutil.h>
#include <qpainter.h>
#include <qpointarray.h>
#include <qstyleplugin.h>

#include <qcombobox.h>
#include <qheader.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qslider.h>
#include <qtabbar.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qobjectlist.h>

#include <kdrawutil.h>
#include <kpixmapeffect.h>
#include <kglobalsettings.h>

#include "qnx.h"

class QnxStylePlugin : public QStylePlugin
{
public:
    QnxStylePlugin() {}
    ~QnxStylePlugin() {}

    QStringList keys() const
		{
			return QStringList() << "Qnx";
		}

    QStyle* create( const QString& key )
		{
			if ( key == "qnx" )
				return new QnxStyle();			
			return 0;
		}
};

Q_EXPORT_PLUGIN( QnxStylePlugin );

QBitmap lightBmp;
QBitmap grayBmp;
QBitmap dgrayBmp;
QBitmap centerBmp;
QBitmap maskBmp;
QBitmap xBmp;
QIntDict<GradientSet> gDict;

static const int itemFrame       = 1;
static const int itemHMargin     = 3;
static const int itemVMargin     = 0;
static const int arrowHMargin    = 6;
static const int rightBorder     = 12;

// ---------------------------------------------------------------------------


GradientSet::GradientSet(const QColor &baseColor)
{
    int i;
    c = baseColor;

    for( i = 0; i < GradientCount; i++ )
        gradients[i] = NULL;
}

GradientSet::~GradientSet()
{
    int i;
    for( i = 0; i < GradientCount; ++i )
    {
        if(gradients[i])
            delete gradients[i];
    }
}

KPixmap* GradientSet::gradient(GradientType type)
{
    int lightValue = 109;
    int darkValue = 108;
    int lightValueRev = 112;
    int darkValueRev = 105;

    if(gradients[type])
        return(gradients[type]);

    switch(type)
    {
        // VERTICAL
        case VSmallest:
            gradients[VSmallest] = new KPixmap;
            gradients[VSmallest]->resize(18, 16);
            KPixmapEffect::gradient(*gradients[VSmallest],
                                    c.light(lightValue-3), c.dark(darkValue),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VSmallestRev:
            gradients[VSmallestRev] = new KPixmap;
            gradients[VSmallestRev]->resize(18, 16);
            KPixmapEffect::gradient(*gradients[VSmallestRev],
                                    c.dark(darkValueRev),
                                    c.light(lightValueRev-3),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VSmall:
            gradients[VSmall] = new KPixmap;
            gradients[VSmall]->resize(18, 24);
            KPixmapEffect::gradient(*gradients[VSmall], c.light(lightValue),
                                    c.dark(darkValue),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VSmallRev:
            gradients[VSmallRev] = new KPixmap;
            gradients[VSmallRev]->resize(18, 24);
            KPixmapEffect::gradient(*gradients[VSmallRev],
                                    c.dark(darkValueRev),
                                    c.light(lightValueRev),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VMed:
            gradients[VMed] = new KPixmap;
            gradients[VMed]->resize(18, 34);
            KPixmapEffect::gradient(*gradients[VMed], c.light(lightValue),
                                    c.dark(darkValue),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VMedRev:
            gradients[VMedRev] = new KPixmap;
            gradients[VMedRev]->resize(18, 34);
            KPixmapEffect::gradient(*gradients[VMedRev], c.dark(darkValueRev),
                                    c.light(lightValueRev),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VLarge:
            gradients[VLarge] = new KPixmap;
            gradients[VLarge]->resize(18, 64);
            KPixmapEffect::gradient(*gradients[VLarge], c.light(lightValue),
                                    c.dark(darkValue),
                                    KPixmapEffect::VerticalGradient);
            break;
        case VLargeRev:
            gradients[VLargeRev] = new KPixmap;
            gradients[VLargeRev]->resize(18, 64);
            KPixmapEffect::gradient(*gradients[VLargeRev],
                                    c.dark(darkValueRev),
                                    c.light(lightValueRev),
                                    KPixmapEffect::VerticalGradient);
            break;
            // HORIZONTAL
        case HSmallest:
            gradients[HSmallest] = new KPixmap;
            gradients[HSmallest]->resize(18, 16);
            KPixmapEffect::gradient(*gradients[HSmallest],
                                    c.light(lightValue-3), c.dark(darkValue),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HSmallestRev:
            gradients[HSmallestRev] = new KPixmap;
            gradients[HSmallestRev]->resize(18, 16);
            KPixmapEffect::gradient(*gradients[HSmallestRev],
                                    c.dark(darkValueRev),
                                    c.light(lightValueRev-3),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HSmall:
            gradients[HSmall] = new KPixmap;
            gradients[HSmall]->resize(18, 24);
            KPixmapEffect::gradient(*gradients[HSmall], c.light(lightValue-3),
                                    c.dark(darkValue),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HSmallRev:
            gradients[HSmallRev] = new KPixmap;
            gradients[HSmallRev]->resize(18, 24);
            KPixmapEffect::gradient(*gradients[HSmallRev], c.dark(darkValueRev),
                                    c.light(lightValueRev-3),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HMed:
            gradients[HMed] = new KPixmap;
            gradients[HMed]->resize(34, 18);
            KPixmapEffect::gradient(*gradients[HMed], c.light(lightValue),
                                    c.dark(darkValue),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HMedRev:
            gradients[HMedRev] = new KPixmap;
            gradients[HMedRev]->resize(34, 18);
            KPixmapEffect::gradient(*gradients[HMedRev], c.dark(darkValueRev),
                                    c.light(lightValueRev),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HLarge:
            gradients[HLarge] = new KPixmap;
            gradients[HLarge]->resize(52, 18);
            KPixmapEffect::gradient(*gradients[HLarge], c.light(lightValue),
                                    c.dark(darkValue),
                                    KPixmapEffect::HorizontalGradient);
            break;
        case HLargeRev:
            gradients[HLargeRev] = new KPixmap;
            gradients[HLargeRev]->resize(52, 18);
            KPixmapEffect::gradient(*gradients[HLargeRev],
                                    c.light(darkValueRev),
                                    c.dark(lightValueRev),
                                    KPixmapEffect::HorizontalGradient);
            break;
        default:
            break;
    }
    return(gradients[type]);
}



// ---------------------------------------------------------------------------

QnxStyle::QnxStyle() 
	: KStyle( AllowMenuTransparency | FilledFrameWorkaround,
              ThreeButtonScrollBar )
{
	highcolor = (QPixmap::defaultDepth() > 8);
	gDict.setAutoDelete(true);
	hoverWidget = 0L;
}


QnxStyle::~QnxStyle()
{
}


void QnxStyle::polish(QWidget* w)
{
    if(qstrcmp(w->name(), "qt_viewport") == 0 || w->testWFlags(WType_Popup) || w->inherits("KDesktop") || w->inherits("PanelButtonBase") || w->isTopLevel())
        return;

    if(w->inherits("QPushButton"))
        w->installEventFilter(this);

    if(w->inherits("QMenuBar") || w->inherits("KToolBarButton"))
    {
        w->setBackgroundMode(QWidget::NoBackground);
        return;
    }

    if(w->parent() && w->parent()->inherits("KToolBar"))
    {
        if(w->inherits("QLabel") || w->inherits("KToolBarSeparator"))
            w->setAutoMask(true);
    }
    else if( w->inherits("QButton") || w->inherits("QComboBox"))
        w->setBackgroundOrigin(QWidget::ParentOrigin);

    if(w->inherits("KToolBar"))
    {
        w->installEventFilter(this);
        w->setBackgroundMode(QWidget::NoBackground);
        return;
    }

}


void QnxStyle::unPolish(QWidget* w)
{
    if(qstrcmp(w->name(), "qt_viewport") == 0 ||w->testWFlags(WType_Popup) || w->inherits("KDesktop")  || w->inherits("PanelButtonBase") || w->isTopLevel())
        return;

    w->setBackgroundMode(QWidget::PaletteBackground);

    if(w->inherits("QPushButton"))
    {
        w->removeEventFilter(this);
    }
    if(w->inherits("QMenuBar") || w->inherits("KToolBarButton"))
    {
        w->setBackgroundMode(QWidget::PaletteBackground);
        return;
    }
    if(w->parent() && w->parent()->inherits("KToolBar"))
    {
        if(w->inherits("QLabel") || w->inherits("KToolBarSeparator"))
            w->setAutoMask(false);
    }
    else if( w->inherits("QButton") || w->inherits("QComboBox"))
        w->setBackgroundOrigin(QWidget::WidgetOrigin);

    if(w->inherits("KToolBar"))
    {
        w->removeEventFilter(this);
        w->setBackgroundMode(QWidget::PaletteBackground);
        return;
    }

}


/* reimp. */
void QnxStyle::renderMenuBlendPixmap( KPixmap& pix, const QColorGroup &cg,
		const QPopupMenu* /* popup */ ) const
{
	QColor col = cg.button();

#ifdef Q_WS_X11 // Only draw menu gradients on TrueColor, X11 visuals
	if ( QPaintDevice::x11AppDepth() >= 24 )
		KPixmapEffect::gradient( pix, col.light(120), col.dark(115),
				KPixmapEffect::HorizontalGradient );
	else
#endif
	pix.fill( col );
}


// This function draws primitive elements as well as their masks.
void QnxStyle::drawPrimitive( PrimitiveElement pe,
									QPainter *p,
									const QRect &r,
									const QColorGroup &cg,
									SFlags flags,
									const QStyleOption& opt ) const
{
	bool down = flags & Style_Down;
	bool on   = flags & Style_On;

	switch(pe)
	{
		// BUTTONS
		// -------------------------------------------------------------------
		case PE_ButtonDefault: {
			break;
		}
			
		case PE_ButtonDropDown:
		case PE_ButtonTool: {
			bool sunken = on || down;
			int  x,y,w,h;
			r.rect(&x, &y, &w, &h);
			int x2 = x+w-1;
			int y2 = y+h-1;
			QPen oldPen = p->pen();

            kDrawBeButton(p, x, y, w, h, cg, sunken,
                          &cg.brush(QColorGroup::Midlight));
			break;
		}

		// PUSH BUTTON
		// -------------------------------------------------------------------
		case PE_ButtonCommand: {
			bool sunken = on || down;
			int  x, y, w, h;
			r.rect(&x, &y, &w, &h);

            int x2 = x+w-1;
            int y2 = y+h-1;
			
			if( highcolor )
			{
				// frame - outermost
                p->setPen(cg.mid().light(110));
                p->drawLine(x, y, x2, y);
                p->drawLine(x, y, x, y2-1);
                p->setPen(cg.mid().light(140));
                p->drawLine(x, y2, x2, y2);
                p->drawLine(x2, y-1, x2, y2);
                // 1st inner frame
                p->setPen(cg.dark().dark(143));
                p->drawRect( x+1, y+1, w-2, h-2 );
                // 2nd inner frame
                p->setPen(sunken ? cg.mid().light(115) : cg.light().light(105));
                p->drawLine(x+2, y+2, x2-2, y+2);
                p->drawLine(x+2, y+2, x+2, y2-2);
                p->setPen(sunken ? cg.light().light(105) : cg.mid().light(115) );
                p->drawLine(x+2, y2-2, x2-2, y2-2);
                p->drawLine(x2-2, y+2, x2-2, y2-2);
                // add the gradient - reverse if sunken
                drawVGradient(p, cg, QColorGroup::Button, x+3, y+3, w-6, h-6, 0, 0, w-6, h-6, sunken );
			}
            else
            {
                if( sunken)
                    kDrawBeButton(p, x, y, w, h, cg, true,
                                  &cg.brush(QColorGroup::Mid));
                else
                    kDrawBeButton(p, x, y, w, h, cg, false,
                                  &cg.brush(QColorGroup::Midlight));
            }
			break;
		}


		// BEVELS
		// -------------------------------------------------------------------
		case PE_ButtonBevel:
        case PE_HeaderSection: {
			int x,y,w,h;
			r.rect(&x, &y, &w, &h);
			bool sunken = on || down;
			int x2 = x+w-1;
			int y2 = y+h-1;

            // frame - outermost
            p->setPen(cg.mid().light(140));
            p->drawLine(x, y2, x2, y2);
            // 1st inner frame - one side only
            p->setPen(cg.dark().dark(143));
            p->drawLine(x, y2-1, x2, y2-1);
            p->drawLine(x2, y, x2, y2-1);
            // 2nd inner frame
            p->setPen(sunken ? cg.mid().light(115) :cg.light().light(105));
            p->drawLine(x, y, x2-1, y);
            p->drawLine(x, y, x, y2-2);
            p->setPen(sunken ? cg.light().light(105) : cg.mid().light(115) );
            p->drawLine(x, y2-2, x2-1, y2-2);
            p->drawLine(x2-1, y, x2-1, y2-2);

            if(highcolor)
                drawVGradient(p, cg, QColorGroup::Button, x+1, y+1, w-4, h-5, 0, 0, w-4, h-5, true); // reverse gradient
            else
                p->fillRect(x+1, y+1, w-4, h-5, cg.brush(QColorGroup::Button));
            
            // last colour determines header text colour
            p->setPen(Qt::black);
			break;
		}


		// FOCUS RECT
		// -------------------------------------------------------------------
		case PE_FocusRect: {
			p->drawWinFocusRect( r );
			break;
		}

		// SCROLLBAR
		// -------------------------------------------------------------------
		case PE_ScrollBarSlider: {
            flags ^= Style_Horizontal;
            
            int x,y,w,h;
			r.rect(&x, &y, &w, &h);
            
            p->setPen(cg.dark());
            p->drawRect(r);
            
            if(highcolor)
            {
                //if( (r.width() > r.height() ) || )
                if( flags & Style_Horizontal )
                    drawHGradient(p, cg, QColorGroup::Button, x+1, y+1,w-2, h-2, 0, 0, w-2, h-2, true);
                else
                    drawVGradient(p, cg, QColorGroup::Button, x+1, y+1,w-2, h-2, 0, 0, w-2, h-2, true);
            }
            else
                p->fillRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2, cg.brush(down ? QColorGroup::Midlight : QColorGroup::Button));

            p->setPen(down ? cg.light() : cg.mid());
            p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
            p->drawLine(r.right()-1, r.top()+1, r.right()-1, r.bottom()-1);
            p->setPen(down ? cg.mid() : cg.light());
            p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
            p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);
            break;
        }

        case PE_ScrollBarAddPage:
		case PE_ScrollBarSubPage: {
            flags ^= Style_Horizontal;
            QColor bgCol( cg.mid() );

            if(r.height())
            {
                // Draw vertical scrollbar part below the slider
                if(flags & Style_Horizontal)
                {
                    // Draw vertical scrollbar part above the slider
                    p->fillRect(r, bgCol);
                    p->setPen(cg.dark());
                    p->drawLine(r.x(), r.y(), r.right(),   r.y());
                    p->drawLine(r.x(), r.bottom(), r.right(),r.bottom());
                }
                else
                {
                    p->fillRect(r, bgCol);
                    p->setPen(cg.dark());
                    p->drawLine(r.x(), r.y(), r.x(), r.bottom());
                    p->drawLine(r.right(), r.y(), r.right(),  r.bottom());
                }
            }
            break;
        }
		case PE_ScrollBarAddLine: {
            
            p->setPen(cg.dark());
            p->drawRect(r);

            if(highcolor)
            {  
                //if( (r.width() > r.height() ) || )
                if( flags & Style_Horizontal )
                    drawHGradient(p, cg, QColorGroup::Button, r.x()+1, r.y()+1,r.width()-2, r.height()-2, 0, 0, r.width()-2, r.height()-2, true);
                else
                    drawVGradient(p, cg, QColorGroup::Button, r.x()+1, r.y()+1,r.width()-2, r.height()-2, 0, 0, r.width()-2, r.height()-2, true);
            }
            else
                p->fillRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2, cg.brush(down ? QColorGroup::Midlight : QColorGroup::Button));
   
            p->setPen(down ? cg.light() : cg.mid());
            p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
            p->drawLine(r.right()-1, r.top()+1, r.right()-1, r.bottom()-1);
            p->setPen(down ? cg.mid() : cg.light());
            p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
            p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);

            drawPrimitive( ((flags & Style_Horizontal) ? PE_ArrowRight : PE_ArrowDown),
                           p, r, cg, flags );
			break;
		}
        

		case PE_ScrollBarSubLine: {
            
            p->setPen(cg.dark());
            p->drawRect(r);

            if(highcolor)
            {  
                //if( (r.width() > r.height() ) || )
                if( flags & Style_Horizontal )
                    drawHGradient(p, cg, QColorGroup::Button, r.x()+1, r.y()+1,r.width()-2, r.height()-2, 0, 0, r.width()-2, r.height()-2, true);
                else
                    drawVGradient(p, cg, QColorGroup::Button, r.x()+1, r.y()+1,r.width()-2, r.height()-2, 0, 0, r.width()-2, r.height()-2, true);
            }
            else
                p->fillRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2, cg.brush(down ? QColorGroup::Midlight : QColorGroup::Button));
   
            p->setPen(down ? cg.light() : cg.mid());
            p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
            p->drawLine(r.right()-1, r.top()+1, r.right()-1, r.bottom()-1);
            p->setPen(down ? cg.mid() : cg.light());
            p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1);
            p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);
            drawPrimitive( ((flags & Style_Horizontal) ? PE_ArrowLeft : PE_ArrowUp),
                           p, r, cg, flags );
			break;
		}


		// CHECKBOX (indicator)
		// -------------------------------------------------------------------
		case PE_Indicator: {
			
			bool enabled  = flags & Style_Enabled;
			bool nochange = flags & Style_NoChange;
			
			int x,y,w,h;
			x=r.x(); y=r.y(); w=r.width(); h=r.height();
			int x2 = x+w-1;
			int y2 = y+h-1;

            p->setPen(cg.mid().light(109));
            p->drawLine(x, y, x2, y);
            p->drawLine(x, y, x, y2-1);
            p->setPen(cg.mid().light(133));
            p->drawLine(x, y2, x2, y2);
            p->drawLine(x2, y-1, x2, y2);
            // 1st inner frame
            p->setPen(cg.dark().dark(143));
            p->drawRect( x+1, y+1, w-2, h-2 );
            
            p->fillRect(x+2, y+2, w-4, h-4, down ? cg.brush(QColorGroup::Background) :   cg.brush(QColorGroup::Light));
            
            // not checked
            if(!on)
            {
                p->setPen(cg.mid().light(110));
                p->drawRect( x+3, y+3, w-6, h-6 );
            }
            // checked
            else
            {
                // draw central rectangle
                p->setPen(cg.mid().light(103));
                p->drawRect( x+5, y+5, 3, 3 );
                // "shadow" diagonal lines
                p->drawLine(x+4, y+3, x2-3, y2-4);
                p->drawLine(x+3, y+4, x2-4, y2-3);
                p->drawLine(x+3, y2-4, x2-4, y+3);
                p->drawLine(x+4, y2-3, x2-3, y+4);
                // draw central dark X
                p->setPen(cg.dark().dark(160));
                p->drawLine(x+3, y+3, x2-3, y2-3);
                p->drawLine(x+3, y2-3, x2-3, y+3);
			}
			break;
		}


		// RADIOBUTTON (exclusive indicator)
		// -------------------------------------------------------------------
		case PE_ExclusiveIndicator: {

            int x,y,w,h;
			r.rect(&x, &y, &w, &h);
            int y2 = y+h-2;
            int x2 = x+w-1;
            if(highcolor)
            {
                drawVGradient(p, cg, QColorGroup::Button, x, y, w, h, 0, 0, w, h, false );
                // remove gradient which is "outside" the circle
                p->fillRect(0, 0, 3, 3, cg.brush(QColorGroup::Background));     // top left
                p->fillRect(9, 0, 3, 3, cg.brush(QColorGroup::Background));     // top right
                p->fillRect(0, 9, 3, 3, cg.brush(QColorGroup::Background));     // bottom left
                p->fillRect(9, 9, 3, 3, cg.brush(QColorGroup::Background));     // bottom right
            }
            else
            {
                // just plain fill
                p->fillRect(x, y, w, h, cg.brush(QColorGroup::Background));
            }

            // longest light line
            p->setPen( cg.background().dark(120) );
            p->drawLine(x,   y+3,  x+3, y);
            p->drawLine(x+8, y, x+11, y+3);
            p->drawLine(x,   y+8,  x+3, y+11);
            p->drawLine(x+8, y+11, x+11, y+8);
            // longer mid line
            p->setPen( cg.mid().dark(105) );
            p->drawLine(x+1,  y+3, x+3,   y+1);
            p->drawLine(x+1,  y+8, x+3,  y+10);
            p->drawLine(x+8,  y+1, x+10,  y+3);
            p->drawLine(x+8, y+10, x+10,  y+8);
            // short darker line
            p->setPen( cg.dark().light(115) );
            p->drawLine(x+1,  y+2, x+2,   y+1);
            p->drawLine(x+1,  y+9, x+2,  y+10);
            p->drawLine(x+9,  y+1, x+10,  y+2);
            p->drawLine(x+9, y+10, x+10,  y+9);
            // surrounding darkest
            p->setPen(cg.dark().light(105));
            p->drawLine(x+4,     y, x+7,    y);
            p->drawLine(x+4,   y2, x+7,   y2);
            p->drawLine(x,    y+4,   x,  y+7);
            p->drawLine(x+11, y+4, x+11, y+7);
            // darkest
            p->setPen(cg.dark().dark(143));
            p->drawLine(x+5,     y, x+6,    y);
            p->drawLine(x+5,   y2, x+6,   y2);
            p->drawLine(x,    y+5,   x,  y+6);
            p->drawLine(x+11, y+5, x+11, y+6);

            p->setPen( cg.dark().light(123) );
            p->drawPoint(  3,  1 ); // top left
            p->drawPoint(  1,  3 );
            p->drawPoint(  8,  1 ); // top right
            p->drawPoint( 10,  3 );
            p->drawPoint(  3, 10 ); // bottom left
            p->drawPoint(  1,  8 );
            p->drawPoint(  8, 10 ); // bottom right
            p->drawPoint( 10,  8 );
            // extra points - light
            p->setPen( cg.mid() );
            p->drawPoint(  4,  1 ); // top
            p->drawPoint(  7,  1 );
            p->drawPoint(  4, 10 ); // bottom
            p->drawPoint(  7, 10 );
            p->drawPoint(  1,  4 ); // left
            p->drawPoint(  1,  7 );
            p->drawPoint( 10,  4 ); // right
            p->drawPoint( 10,  7 );
            p->setPen( Qt::red );

   // extra shadows
            p->setPen( cg.background() );
            p->drawLine(x, y+h, x2, y+h);
            p->setPen( cg.background().dark(105) );
            p->drawLine(x+3, y+h, x+8, y+h);
            p->setPen( cg.background().dark(110) );
            p->drawLine(x+4, y+h, x+7, y+h);

   // Is this radio button selected ?
            if( on )
            {
                // if so, draw centre dot...
                p->fillRect(x+4, x+4, 4, 4, cg.dark().light(120));
                p->fillRect(x+5, x+5, 2, 2, cg.dark().light(103));  // innermost
                // next darkest
                p->setPen( cg.dark().light(110) );
                p->drawLine(x+4, y+5, x+4, y+6 ); // left
                p->drawLine(x+7, y+5, x+7, y+6 ); // right
                p->drawLine(x+5, y+4, x+6, y+4 ); // top
                p->drawLine(x+5, y+7, x+6, y+7 ); // bottom
                // mid
                p->setPen( cg.mid().dark(125) );
                p->drawLine(x+3, y+5, x+3, y+6 ); // left
                p->drawLine(x+8, y+5, x+8, y+6 ); // right
                p->drawLine(x+5, y+3, x+6, y+3 ); // top
                p->drawLine(x+5, y+8, x+6, y+8 ); // bottom
                // mid light
                p->setPen( cg.mid() );
                p->drawPoint( x+4, y+3);
                p->drawPoint( x+7, y+3);
                p->drawPoint( x+4, y+8);
                p->drawPoint( x+7, y+8);
                p->drawPoint( x+3, y+4);
                p->drawPoint( x+3, y+7);
                p->drawPoint( x+8, y+4);
                p->drawPoint( x+8, y+7);
                // light
                p->setPen( cg.mid().light(115) );
                p->drawPoint( x+3, y+3);
                p->drawPoint( x+3, y+8);
                p->drawPoint( x+8, y+3);
                p->drawPoint( x+8, y+8);
                // lightest
                p->setPen( cg.mid().light(130) );
                p->drawLine(x+2, y+5, x+2, y+6 );      // left
                p->drawLine(x+9, y+5, x+9, y+6 );      // right
                p->drawLine(x+5, y+2, x+6, y+2 );      // top
                p->drawLine(x+5, y+9, x+6, y+9 );      // top
            }
			break;
		}


		// RADIOBUTTON (exclusive indicator) mask
		// -------------------------------------------------------------------
		case PE_ExclusiveIndicatorMask: {
            int x,y,w,h;
			r.rect(&x, &y, &w, &h);
            int y2 = y+h-1;
            int x2 = x+w-1;
            // last line is used for shadows
            h--;
    
            p->fillRect(x, y, w, h, Qt::color0);
    
            p->setPen(Qt::color1);
    
            p->drawLine(x+4, y,   x2-3, y );
            p->drawLine(x+3, y+1, x2-2, y+1 );
            p->drawLine(x+2, y+2, x2-1, y+2 );
            p->fillRect(x+1, y+3, w, h-6, Qt::color1);
            p->drawLine(x+2, y2-3, x2-1, y2-3 );
            p->drawLine(x+3, y2-2, x2-2, y2-2 );
            p->drawLine(x+4, y2-1, x2-3, y2-1 );

			break;
		}


		// SPLITTER/DOCKWINDOW HANDLES
		// -------------------------------------------------------------------
		case PE_DockWindowResizeHandle:
		case PE_Splitter: {
			int x,y,w,h;
			r.rect(&x, &y, &w, &h);
			int x2 = x+w-1;
			int y2 = y+h-1;

			p->setPen(cg.dark());
			p->drawRect(x, y, w, h);
			p->setPen(cg.background());
			p->drawPoint(x, y);
			p->drawPoint(x2, y);
			p->drawPoint(x, y2);
			p->drawPoint(x2, y2);
			p->setPen(cg.light());
			p->drawLine(x+1, y+1, x+1, y2-1);
			p->drawLine(x+1, y+1, x2-1, y+1);
			p->setPen(cg.midlight());
			p->drawLine(x+2, y+2, x+2, y2-2);
			p->drawLine(x+2, y+2, x2-2, y+2);
			p->setPen(cg.mid());
			p->drawLine(x2-1, y+1, x2-1, y2-1);
			p->drawLine(x+1, y2-1, x2-1, y2-1);
			p->fillRect(x+3, y+3, w-5, h-5, cg.brush(QColorGroup::Background));
			break;
		}


		// GENERAL PANELS
		// -------------------------------------------------------------------
		case PE_Panel:
		case PE_PanelPopup:
        case PE_WindowFrame:
		case PE_PanelLineEdit: {
			bool sunken  = flags & Style_Sunken;
			int x,y,w,h;
			r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
            
            QPen oldPen = p->pen();
            
        //      // frame - outermost
              p->setPen(cg.mid().light(110));
              p->drawLine(x, y, x2, y);
              p->drawLine(x, y, x, y2-1);
              p->setPen(cg.mid().light(140));
              p->drawLine(x, y2, x2, y2);
              p->drawLine(x2, y-1, x2, y2);
            // 1st inner frame
              p->setPen(cg.dark().dark(143));
              p->drawRect( x+1, y+1, w-2, h-2 );
              // 2nd inner frame
//              p->setPen(sunken ? cg.mid().light(115) : cg.light());
//              p->drawLine(x+2, y+2, x2-2, y+2);
//              p->drawLine(x+2, y+2, x+2, y2-2);
//              p->setPen(sunken ? cg.light() : cg.mid().light(115) );
//              p->drawLine(x+2, y2-2, x2-2, y2-2);
//              p->drawLine(x2-2, y+2, x2-2, y2-2);
            
            p->setPen(oldPen);
			break;
		}


		// MENU / TOOLBAR PANEL
		// -------------------------------------------------------------------
		case PE_PanelMenuBar: 			// Menu
		case PE_PanelDockWindow: {		// Toolbar
			int x2 = r.x()+r.width()-1;
			int y2 = r.y()+r.height()-1;
            int x,y,w,h;
			r.rect(&x, &y, &w, &h);

			if(highcolor)
            {
                qDrawShadePanel(p, x, y, w, h, cg, false, 1);
                if(w > h)
                    drawVGradient(p, cg, QColorGroup::Background, x+1, y+1, w-2, h-2, 0, 0, w-2, h-2, true);
                else
                    drawHGradient(p, cg, QColorGroup::Background, x+1, y+1, w-2, h-2, 0, 0, w-2, h-2, true);
            }
            else
                qDrawShadePanel(p, x, y, w, h, cg, false, 1, &cg.brush(QColorGroup::Background));
            
            //extra line at the bottom
            p->setPen(cg.mid());
            p->drawLine(x, y+h-2, x+w, y+h-2);


			break;
		}
		default:
		{
			// ARROWS
			// -------------------------------------------------------------------
			if (pe >= PE_ArrowUp && pe <= PE_ArrowLeft)
			{
                QPointArray a;

                switch ( pe ) {
                    case PE_ArrowUp:
                        a.setPoints( 7, -4,1, 2,1, -3,0, 1,0, -2,-1, 0,-1, -1,-2 );
                        break;

                    case PE_ArrowDown:
                        a.setPoints( 7, -4,-2, 2,-2, -3,-1, 1,-1, -2,0, 0,0, -1,1 );
                        break;

                    case PE_ArrowRight:
                        a.setPoints( 7, -2,-3, -2,3, -1,-2, -1,2, 0,-1, 0,1, 1,0 );
                        break;

                    case PE_ArrowLeft:
                        a.setPoints( 7, 0,-3, 0,3, -1,-2, -1,2, -2,-1, -2,1, -3,0 );
                        break;

                    default:
                        break;
                }

                if (a.isNull())
                    return;

                p->save();
                if ( flags & Style_Enabled ) {
                    a.translate( r.x() + r.width() / 2, r.y() + r.height() / 2 );
                    p->setPen( cg.buttonText() );
                    p->drawLineSegments( a, 0, 3 );         // draw arrow
                    p->drawPoint( a[6] );
                } else {
                    a.translate( r.x() + r.width() / 2 + 1, r.y() + r.height() / 2 + 1 );
                    p->setPen( cg.light() );
                    p->drawLineSegments( a, 0, 3 );         // draw arrow
                    p->drawPoint( a[6] );
                    a.translate( -1, -1 );
                    p->setPen( cg.mid() );
                    p->drawLineSegments( a, 0, 3 );         // draw arrow
                    p->drawPoint( a[6] );
                }
                p->restore(); 
	
			} else
				KStyle::drawPrimitive( pe, p, r, cg, flags, opt );
		}
    }
}

void QnxStyle::drawKStylePrimitive( KStylePrimitive kpe,
    QPainter* p,
    const QWidget* widget,
    const QRect &r,
    const QColorGroup &cg,
    SFlags flags,
    const QStyleOption &opt ) const
{
    switch ( kpe )
    {
        // TOOLBAR HANDLE
        // -------------------------------------------------------------------
        case KPE_ToolBarHandle:
        case KPE_GeneralHandle: {
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            if(h > w)
            {
                if(highcolor)
                    drawVGradient(p, cg, QColorGroup::Background, x, y, w, h, 0, 0, w, h, true);
                else
                    p->fillRect(x, y, w, h, cg.brush(QColorGroup::Background));
                int x2 = x+w-1;
                int y2 = y+h-1;

                p->setPen(cg.light());
                p->drawLine(x, y, x, y2-1);
                p->drawPoint( x+1, y );
                p->drawLine(x+3, y, x+3, y2);
                p->drawPoint( x+4, y );
                p->drawLine(x+6, y, x+6, y2);
                p->drawPoint( x+7, y );
                p->drawLine(x+9, y, x+9, y2);

                p->setPen(cg.mid());
                p->drawPoint( x+1, y2 );
                p->drawPoint( x+2, y2 );
                p->drawLine(x+2, y, x+2, y2);
                p->drawPoint( x+4, y2 );
                p->drawLine(x+5, y, x+5, y2);
                p->drawPoint( x+7, y2 );
                p->drawLine(x+8, y, x+8, y2);

                p->setPen(cg.dark());
                p->drawLine(x, y2+1, x2, y2+1);

            }
            // horizontal  ----
            else
            {
                if(highcolor)
                    drawVGradient(p, cg, QColorGroup::Background, x, y, w, h, 0, 0, w, h, true);
                else
                    p->fillRect(x, y, w, h, cg.brush(QColorGroup::Background));
                int x2 = x+w-2;
                int y2 = y+h-1;

                p->setPen(cg.light());
                p->drawLine(x, y, x2-1, y);
                p->drawPoint( x, y+1 );
                p->drawLine(x, y+3, x2, y+3);
                p->drawPoint( x, y+4 );
                p->drawLine(x, y+6, x2, y+6);
                p->drawPoint( x, y+7 );
                p->drawLine(x, y+9, x2, y+9);

                p->setPen(cg.mid());
                p->drawPoint( x, y+1 );
                p->drawPoint( x, y+2 );
                p->drawLine(x, y+2, x2, y+2);
                p->drawPoint( x, y+4 );
                p->drawLine(x, y+5, x2, y+5);
                p->drawPoint( x, y+7 );
                p->drawLine(x, y+8, x2, y+8);

                p->setPen(cg.dark());
                p->drawLine(x2+1, y, x2+1, y2);
                break;
            }
            break;
        }
							   
        case KPE_SliderGroove: {
            const QSlider* slider = (const QSlider*)widget;
            bool horizontal = slider->orientation() == Horizontal;
            int gcenter = (horizontal ? r.height() : r.width()) / 2;

            QRect gr;
            if (horizontal)
                gr = QRect(r.x(), r.y()+gcenter-3, r.width(), 7);
            else
                gr = QRect(r.x()+gcenter-3, r.y(), 7, r.height());

            int x,y,w,h;
            gr.rect(&x, &y, &w, &h);
            int x2=x+w-1;
            int y2=y+h-1;

            // Draw the slider groove.
            p->setPen(cg.dark());
            p->drawLine(x+2, y, x2-2, y);
            p->drawLine(x, y+2, x, y2-2);
            p->fillRect(x+2,y+2,w-4, h-4, 
                        slider->isEnabled() ? cg.dark() : cg.mid());
            p->setPen(cg.shadow());
            p->drawRect(x+1, y+1, w-2, h-2);
            p->setPen(cg.light());
            p->drawPoint(x+1,y2-1);
            p->drawPoint(x2-1,y2-1);
            p->drawLine(x2, y+2, x2, y2-2);
            p->drawLine(x+2, y2, x2-2, y2);
            break;
        }

        // SLIDER HANDLE
        // -------------------------------------------------------------------
        case KPE_SliderHandle: {
            const QSlider* slider = (const QSlider*)widget;
            bool horizontal = slider->orientation() == Horizontal;
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
				

            // frame - outermost
            // 1st inner frame
            p->setPen(cg.dark().dark(143));
            p->drawRect( x+1, y+1, w-2, h-2 );
            // 2nd inner frame
            p->setPen(cg.light().light(105));
            p->drawLine(x+2, y+2, x2-2, y+2);
            p->drawLine(x+2, y+2, x+2, y2-2);
            p->setPen( cg.mid().light(115) );
            p->drawLine(x+2, y2-2, x2-2, y2-2);
            p->drawLine(x2-2, y+2, x2-2, y2-2);
            // FILL
            if(highcolor)
                drawVGradient(p, cg, QColorGroup::Button, x+3, y+3, w-6, h-6, 0, 0, w-6, h-6,false);
            else
                p->fillRect(x+3, y+3, w-6, h-6, cg.background());

            // SHADOW
            p->setPen(cg.dark().light(125));
            // right
            p->drawLine(x2, y+2, x2, y2);
            // bottom
            p->drawLine(x+2, y2, x2, y2);

            break;
        }

        default:
            KStyle::drawKStylePrimitive( kpe, p, widget, r, cg, flags, opt);
    }
}

void QnxStyle::drawControl( ControlElement element,
								  QPainter *p,
								  const QWidget *widget,
								  const QRect &r,
								  const QColorGroup &cg,
								  SFlags flags,
								  const QStyleOption& opt ) const
{
	switch (element)
	{
		// PUSHBUTTON
		// -------------------------------------------------------------------
		case CE_PushButton: {
			if ( widget == hoverWidget )
				flags |= Style_MouseOver;
			
            drawPrimitive( PE_ButtonCommand, p, r, cg, flags );
			
			break;
		}

							   
		// PUSHBUTTON LABEL
		// -------------------------------------------------------------------
		case CE_PushButtonLabel: {
			const QPushButton* button = (const QPushButton*)widget;
			bool active = button->isOn() || button->isDown();
			int x, y, w, h;
			r.rect( &x, &y, &w, &h );

			// Shift button contents if pushed.
			if ( active ) {
				x += pixelMetric(PM_ButtonShiftHorizontal, widget); 
				y += pixelMetric(PM_ButtonShiftVertical, widget);
				flags |= Style_Sunken;
			}

			// Does the button have a popup menu?
			if ( button->isMenuButton() ) {
				int dx = pixelMetric( PM_MenuButtonIndicator, widget );
				drawPrimitive( PE_ArrowDown, p, QRect(x + w - dx - 2, y + 2, dx, h - 4),
							   cg, flags, opt );
				w -= dx;
			}

			// Draw the icon if there is one
			if ( button->iconSet() && !button->iconSet()->isNull() ) {
				QIconSet::Mode  mode  = QIconSet::Disabled;
				QIconSet::State state = QIconSet::Off;

				if (button->isEnabled())
					mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
				if (button->isToggleButton() && button->isOn())
					state = QIconSet::On;

				QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );
				p->drawPixmap( x + 4, y + h / 2 - pixmap.height() / 2, pixmap );
				int  pw = pixmap.width();
				x += pw + 4;
				w -= pw + 4;
			}

			// Make the label indicate if the button is a default button or not
			if ( active || button->isDefault() ) {
				// Draw "fake" bold text  - this enables the font metrics to remain
				// the same as computed in QPushButton::sizeHint(), but gives
				// a reasonable bold effect.
				int i;

				// Text shadow
				if (button->isEnabled()) // Don't draw double-shadow when disabled
					for(i=0; i<2; i++)
						drawItem( p, QRect(x+i+1, y+1, w, h), AlignCenter | ShowPrefix, 
								button->colorGroup(), button->isEnabled(), button->pixmap(),
								button->text(), -1,	
								&button->colorGroup().mid() );

				// Normal Text
				for(i=0; i<2; i++)
					drawItem( p, QRect(x+i, y, w, h), AlignCenter | ShowPrefix, 
							button->colorGroup(), button->isEnabled(), button->pixmap(),
							button->text(), -1,
							&button->colorGroup().buttonText() );
			} else
				drawItem( p, QRect(x, y, w, h), AlignCenter | ShowPrefix, button->colorGroup(),
						button->isEnabled(), button->pixmap(), button->text(), -1,
						&button->colorGroup().buttonText() );

			// Draw a focus rect if the button has focus
			if ( flags & Style_HasFocus )
				drawPrimitive( PE_FocusRect, p,
						QStyle::visualRect(subRect(SR_PushButtonFocusRect, widget), widget),
						cg, flags );
			break;
		}


		// MENUBAR ITEM (sunken panel on mouse over)
		// -------------------------------------------------------------------
		case CE_MenuBarItem:
		{
            QMenuBar  *mb = (QMenuBar*)widget;
            QMenuItem *mi = opt.menuItem();
            QRect      pr = mb->rect();
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
			int x2 = x+w-1;
			int y2 = y+h-1;


            bool active  = flags & Style_Active;
            bool focused = flags & Style_HasFocus;

            if ( active && focused )
                qDrawShadePanel(p, r.x(), r.y(), r.width(), r.height(),
                                cg, true, 1, &cg.brush(QColorGroup::Midlight));
            else
                drawVGradient(p, cg, QColorGroup::Background, x, y, w, h, 0, 0, w, h, true);
            drawItem( p, r, AlignCenter | AlignVCenter | ShowPrefix
                    | DontClip | SingleLine, cg, flags & Style_Enabled,
                    mi->pixmap(), mi->text() );

            break;
		}


		// POPUPMENU ITEM
		// -------------------------------------------------------------------
		case CE_PopupMenuItem: {
            const QPopupMenu *popupmenu = (const QPopupMenu *) widget;
    
            QMenuItem *mi = opt.menuItem();
            if ( !mi ) {
                // Don't leave blank holes if we set NoBackground for the QPopupMenu.
                // This only happens when the popupMenu spans more than one column.
                if (! (widget->erasePixmap() && !widget->erasePixmap()->isNull()) )
                    p->fillRect(r, cg.brush(QColorGroup::Button) );
                break;
            }       
                    
            int  tab        = opt.tabWidth();
            int  checkcol   = opt.maxIconWidth();
            bool enabled    = mi->isEnabled();
            bool checkable  = popupmenu->isCheckable();
            bool active     = flags & Style_Active;
            bool etchtext   = styleHint( SH_EtchDisabledText );
            bool reverse    = false;
            int x, y, w, h;
            r.rect( &x, &y, &w, &h ); 
        
            if ( checkable )
                checkcol = QMAX( checkcol, 20 );
                            
            // Are we a menu item separator?
            if ( mi->isSeparator() ) {
                p->setPen( cg.dark() );
                p->drawLine( x, y, x+w, y );
                p->setPen( cg.light() );
                p->drawLine( x, y+1, x+w, y+1 );
                break;
            }       
                        
            // Draw the menu item background
            if ( active )
                qDrawShadePanel( p, x, y, w, h, cg, true, 1,
                                 &cg.brush(QColorGroup::Midlight) );
            // Draw the transparency pixmap
            else if ( widget->erasePixmap() && !widget->erasePixmap()->isNull() )
                p->drawPixmap( x, y, *widget->erasePixmap(), x, y, w, h );
            // Draw a solid background
            else
                p->fillRect( r, cg.button() );

            // Do we have an icon?
            if ( mi->iconSet() ) {
                QIconSet::Mode mode;
                QRect cr = visualRect( QRect(x, y, checkcol, h), r );
                
                // Select the correct icon from the iconset
                if ( active )
                    mode = enabled ? QIconSet::Active : QIconSet::Disabled;
                else                
                    mode = enabled ? QIconSet::Normal : QIconSet::Disabled;
                     
                // Do we have an icon and are checked at the same time?
                // Then draw a "pressed" background behind the icon
                if ( checkable && !active && mi->isChecked() )
                    qDrawShadePanel( p, cr.x(), cr.y(), cr.width(), cr.height(),
                                     cg, true, 1, &cg.brush(QColorGroup::Midlight) );
                QPixmap pixmap = mi->iconSet()->pixmap( QIconSet::Small, mode );
                QRect pmr( 0, 0, pixmap.width(), pixmap.height() );
                pmr.moveCenter( cr.center() );
                p->drawPixmap( pmr.topLeft(), pixmap );
            }

            // Are we checked? (This time without an icon)
            else if ( checkable && mi->isChecked() ) {
                int cx = reverse ? x+w - checkcol : x;

                // We only have to draw the background if the menu item is inactive -
                // if it's active the "pressed" background is already drawn
                if ( ! active )
                    qDrawShadePanel( p, cx, y, checkcol, h, cg, true, 1,
                                     &cg.brush(QColorGroup::Midlight) );

                // Draw the checkmark
                SFlags cflags = Style_Default;
                cflags |= active ? Style_Enabled : Style_On;

                drawPrimitive( PE_CheckMark, p, QRect( cx + itemFrame, y + itemFrame,
                                                       checkcol - itemFrame*2, h - itemFrame*2), cg, cflags );
            }

            // Time to draw the menu item label...
            int xm = itemFrame + checkcol + itemHMargin; // X position margin

            int xp = reverse ? // X position
                x + tab + rightBorder + itemHMargin + itemFrame - 1 :
                x + xm;

            int offset = reverse ? -1 : 1;  // Shadow offset for etched text

            // Label width (minus the width of the accelerator portion)
            int tw = w - xm - tab - arrowHMargin - itemHMargin * 3 - itemFrame + 1;

            // Set the color for enabled and disabled text 
            // (used for both active and inactive menu items)
            p->setPen( enabled ? cg.buttonText() : cg.mid() );

            // This color will be used instead of the above if the menu item
            // is active and disabled at the same time. (etched text)
            QColor discol = cg.mid();

            // Does the menu item draw it's own label?
            if ( mi->custom() ) {
                int m = itemVMargin;
                // Save the painter state in case the custom
                // paint method changes it in some way
                p->save();

                // Draw etched text if we're inactive and the menu item is disabled
                if ( etchtext && !enabled && !active ) {
                    p->setPen( cg.light() );
                    mi->custom()->paint( p, cg, active, enabled, xp+offset, y+m+1, tw, h-2*m );
                    p->setPen( discol );
                }
                mi->custom()->paint( p, cg, active, enabled, xp, y+m, tw, h-2*m );
                p->restore();
            }
            else {
                // The menu item doesn't draw it's own label
                QString s = mi->text();

                // Does the menu item have a text label?
                if ( !s.isNull() ) {
                    int t = s.find( '\t' );
                    int m = itemVMargin;
                    int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;
                    text_flags |= reverse ? AlignRight : AlignLeft;

                    // Does the menu item have a tabstop? (for the accelerator text)
                    if ( t >= 0 ) {
                        int tabx = reverse ? x + rightBorder + itemHMargin + itemFrame :
                            x + w - tab - rightBorder - itemHMargin - itemFrame;

                        // Draw the right part of the label (accelerator text)
                        if ( etchtext && !enabled && !active ) {
                            // Draw etched text if we're inactive and the menu item is disabled
                            p->setPen( cg.light() );
                            p->drawText( tabx+offset, y+m+1, tab, h-2*m, text_flags, s.mid( t+1 ) );
                            p->setPen( discol );
                        }
                        p->drawText( tabx, y+m, tab, h-2*m, text_flags, s.mid( t+1 ) );
                        s = s.left( t );
                    }

                    // Draw the left part of the label (or the whole label 
                    // if there's no accelerator)
                    if ( etchtext && !enabled && !active ) {
                        // Etched text again for inactive disabled menu items...
                        p->setPen( cg.light() );
                        p->drawText( xp+offset, y+m+1, tw, h-2*m, text_flags, s, t );
                        p->setPen( discol );
                    }

                    p->drawText( xp, y+m, tw, h-2*m, text_flags, s, t );

                }

                // The menu item doesn't have a text label
                // Check if it has a pixmap instead
                else if ( mi->pixmap() ) {
                    QPixmap *pixmap = mi->pixmap();

                    // Draw the pixmap
                    if ( pixmap->depth() == 1 )
                        p->setBackgroundMode( OpaqueMode );

                    int diffw = ( ( w - pixmap->width() ) / 2 )
                        + ( ( w - pixmap->width() ) % 2 );
                    p->drawPixmap( x+diffw, y+itemFrame, *pixmap );

                    if ( pixmap->depth() == 1 )
                        p->setBackgroundMode( TransparentMode );
                }
            }
            // Does the menu item have a submenu?
                  if ( mi->popup() ) {
                      PrimitiveElement arrow = reverse ? PE_ArrowLeft : PE_ArrowRight;
                      int dim = (h-2*itemFrame) / 2;
                      QRect vr = visualRect( QRect( x + w - arrowHMargin - itemFrame - dim,
                                                    y + h / 2 - dim / 2, dim, dim), r );

                      // Draw an arrow at the far end of the menu item
                      if ( active ) {
                          if ( enabled )
                              discol = cg.buttonText();

                          QColorGroup g2( discol, cg.highlight(), white, white,
                                          enabled ? white : discol, discol, white );

                          drawPrimitive( arrow, p, vr, g2, Style_Enabled );
                      } else
                          drawPrimitive( arrow, p, vr, cg,
                                         enabled ? Style_Enabled : Style_Default );
                  }
                  break;
        
        }
        case CE_TabBarTab: {
            const QTabBar* tb = ( const QTabBar* ) widget;
            QTabBar::Shape tbs = tb->shape();
            bool selected = flags & Style_Selected;
            const QTabBar *tabBar = (const QTabBar *) widget;
            QRect r2 = QRect(r.x(), r.y(), r.width(), r.height());
            r2.setY(r.y()+1);
            r2.setWidth(r.width()+1);

            // only do special draw for top tabs
            if(tbs != QTabBar::RoundedAbove)
            {  
                //KStyle::drawTab(p, tabBar, tab, selected);
                return;
            }

            int yBottom = r2.bottom();
            int gap = 3;
            int yTop;
            int xLeft = r2.x() + 1 + gap;
            int xRight = r2.right()-gap;
    
            if ( selected )
                yTop = r2.y() + 1;
            else 
                yTop = r2.y() + 2;

   // [0.1 tab gradient fill]
   /*if( selected )
      drawVGradient(p, g, QColorGroup::Light, xLeft+1, yTop+2, r.width()-(gap*2)-2, r.height()-(yTop+2), 2, 2, r.width()-(gap*2), r.height()-(yTop), true ); 
   else
      drawVGradient(p, g, QColorGroup::Mid, xLeft+1, yTop+2, r.width()-(gap*2)-2, r.height()-(yTop+2), 2, 2, r.width()-(gap*2), r.height()-(yTop), true );
   */

   // [0.2] Changed appearance of tabs to blend in with tab panel
            p->fillRect( xLeft+1, yTop+2, r2.width()-(gap*2)-2, r2.height()-(yTop+2), cg.brush(QColorGroup::Background) );

            // INSET
            p->setPen( cg.light().light(115));
            // left gap
            p->drawLine(xLeft-gap-1, yBottom+1,  xLeft-1, yBottom+1);
            // left
            p->drawLine( xLeft+1, yTop+1, xLeft+1, yBottom);
            // top
            p->drawLine(xLeft+1, yTop+1, xRight-2, yTop+1);
            // right gap
            p->drawLine(xRight, yBottom+1,  xRight+gap, yBottom+1);
            // right
            p->setPen( cg.mid().light(135) );
            p->drawLine(xRight-1, yTop+1, xRight-1, yBottom);

   // NORMAL
            p->setPen(cg.dark().dark(143));
            // left gap
            p->drawLine(xLeft-gap-1, yBottom,  xLeft-1, yBottom);
            // left
            p->drawLine(xLeft, yTop, xLeft, yBottom);
            // top
            p->drawLine(xLeft, yTop,  xRight-2, yTop);
            // right
            p->drawLine(xRight-1, yTop+1, xRight-1, yBottom);
            // right gap
            p->drawLine(xRight, yBottom,  xRight+gap, yBottom);
// RIGHT DESIGN
            // only for larger tabs
            if( r2.width() > 35 )
            {
                for( int dx = xRight-10; dx < xRight-1; dx+=3  )
                {
                    p->setPen(cg.mid() );
                    p->drawLine(dx, yTop+1, dx, yBottom);
                    p->drawLine(dx+1, yBottom, dx+2, yBottom);
                    p->setPen( cg.light().light(110));
                    p->drawLine(dx+1, yTop+1, dx+1, yBottom-1);
                    p->drawPoint( dx+2, yTop+1 );
                }
            }

            if( !selected )
            {
                p->setPen(cg.dark().dark(143));
                p->drawLine(xLeft-gap-1, yBottom,  xRight+gap, yBottom);
                p->setPen( cg.light().light(115));
                p->drawLine(xLeft-gap-1, yBottom+1,  xRight+gap, yBottom+1);
            }
            else
            {
                // SHADOW
                p->setPen(cg.dark().light(125));
                // right
                p->drawLine(xRight, yTop+1, xRight, yBottom-1);
                p->drawPoint( xRight-1, yTop );
            }
            break;
        }
        default:
			KStyle::drawControl(element, p, widget, r, cg, flags, opt);
	}
}


void QnxStyle::drawControlMask( ControlElement element,
								  	  QPainter *p,
								  	  const QWidget *widget,
								  	  const QRect &r,
								  	  const QStyleOption& opt ) const
{
	switch (element)
	{
		// PUSHBUTTON MASK
		// ----------------------------------------------------------------------
		case CE_PushButton: {
			int x1, y1, x2, y2;
			r.coords( &x1, &y1, &x2, &y2 );
			QCOORD corners[] = { x1,y1, x2,y1, x1,y2, x2,y2 };
			p->fillRect( r, color1 );
			p->setPen( color0 );
			p->drawPoints( QPointArray(4, corners) );
			break;
		}

		default:
			KStyle::drawControlMask(element, p, widget, r, opt);
	}
}


void QnxStyle::drawComplexControl( ComplexControl control,
                                         QPainter *p,
                                         const QWidget *widget,
                                         const QRect &r,
                                         const QColorGroup &cg,
                                         SFlags flags,
									     SCFlags controls,
									     SCFlags active,
                                         const QStyleOption& opt ) const
{
    bool down = flags & Style_Down;
    bool on = flags & Style_On;

	switch(control)
	{
		// COMBOBOX
		// -------------------------------------------------------------------
		case CC_ComboBox: {
            QPen oldPen = p->pen();
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
            
            if ( controls & SC_ComboBoxArrow ) {
                bool sunken = ( active == SC_ComboBoxArrow );
                if(highcolor)
                {
                    // frame - outermost
                    p->setPen(cg.mid().light(110));
                    p->drawLine(x, y, x2, y);
                    p->drawLine(x, y, x, y2-1);
                    p->setPen(cg.mid().light(140));
                    p->drawLine(x, y2, x2, y2);
                    p->drawLine(x2, y-1, x2, y2);
                    // 1st inner frame
                    p->setPen(cg.dark().dark(143));
                    p->drawRect( x+1, y+1, w-2, h-2 );
                    // 2nd inner frame
                    p->setPen(sunken ? cg.mid().light(115) : cg.light().light(105));
                    p->drawLine(x+2, y+2, x2-2, y+2);
                    p->drawLine(x+2, y+2, x+2, y2-2);
                    p->setPen(sunken ? cg.light().light(105) : cg.mid().light(115) );
                    p->drawLine(x+2, y2-2, x2-2, y2-2);
                    p->drawLine(x2-2, y+2, x2-2, y2-2);
                    // add the gradient - reverse if sunken
                    drawVGradient(p, cg, QColorGroup::Button, x+3, y+3, w-6, h-6, 0, 0, w-6, h-6, sunken );
                }
                else
                {
                    // just plain
                    if( sunken)
                        kDrawBeButton(p, x, y, w, h, cg, true, &cg.brush(QColorGroup::Mid));
                    else
                        kDrawBeButton(p, x, y, w, h, cg, sunken, &cg.brush(QColorGroup::Midlight));
                }

                // Now draw the arrow
                int arrow_h = h / 3;
                int arrow_w = arrow_h;
                int arrow_x = w - arrow_w - 6;
                int arrow_y = (h - arrow_h) / 2;
                QRect ar = QRect(arrow_x, arrow_y, arrow_w, arrow_h);
                drawPrimitive(PE_ArrowDown, p, ar, cg, flags);
                int lineX = x + arrow_x - 6;
                if( lineX > 5 )
                {
                    p->setPen(cg.mid().dark(108));
                    p->drawLine(lineX, y+2, lineX, y2-3);
                    p->setPen(cg.mid().light(115));
                    p->drawLine(lineX+1, y+2, lineX+1, y2-3);
                }
            }
            if ( controls & SC_ComboBoxEditField )
            {
                  const QComboBox * cb = (const QComboBox *) widget;

                  if ( cb->hasFocus() && !cb->editable() ) {
                    QRect re = QStyle::visualRect(
                                subRect(SR_ComboBoxFocusRect, cb), widget);

                    drawPrimitive( PE_FocusRect, p, re, cg,
                                   Style_FocusAtBorder, QStyleOption(cg.highlight()));
                  }
            }
            p->setPen(cg.foreground());
            break;
        }
        case CC_SpinWidget: {
            const QSpinWidget *spinwidget = (const QSpinWidget *) widget;
            QRect frame, up, down;

            frame = querySubControlMetrics(CC_SpinWidget, widget,
                                           SC_SpinWidgetFrame, opt);
            up = spinwidget->upRect();
            down = spinwidget->downRect();

            if ((controls & SC_SpinWidgetFrame) && frame.isValid()) {
                drawPrimitive( PE_Panel, p, frame, cg, flags | Style_Sunken );
                
            }
            

            if ((controls & SC_SpinWidgetUp) && up.isValid()) {
                PrimitiveElement pe = PE_SpinWidgetUp;
                if ( spinwidget->buttonSymbols() == QSpinWidget::PlusMinus )
                    pe = PE_SpinWidgetPlus;

                p->setPen(cg.mid().dark(108));
                p->drawLine( up.topLeft(), up.bottomLeft() );
                p->setPen(cg.mid().light(115));
                p->drawLine( up.x() + 1, up.y(),
                             up.x() + 1, up.y() + up.height());
                
                up.addCoords( 1, 0, 0, 0 );
                p->fillRect( up, cg.brush( QColorGroup::Button ) );

                up.addCoords( 1, 0, 0, 0 );
                drawPrimitive(pe, p, up, cg, flags |
                              ((active == SC_SpinWidgetUp) ?
                               Style_On | Style_Sunken : Style_Raised));
            }

            if ((controls & SC_SpinWidgetDown) && down.isValid()) {
                PrimitiveElement pe = PE_SpinWidgetDown;
                if ( spinwidget->buttonSymbols() == QSpinWidget::PlusMinus )
                    pe = PE_SpinWidgetMinus;

                p->setPen(cg.mid().dark(108));
                p->drawLine( down.topLeft(), down.bottomLeft() );
                p->setPen(cg.mid().light(115));
                p->drawLine( down.x() + 1, down.y(),
                             down.x() + 1, down.y() + down.height());

                
                down.addCoords( 1, 0, 0, 0 );
                p->fillRect( down, cg.brush( QColorGroup::Button ) );

                down.addCoords( 1, 0, 0, 0 );
                drawPrimitive(pe, p, down, cg, flags |
                              ((active == SC_SpinWidgetDown) ?
                               Style_On | Style_Sunken : Style_Raised));
            }

            break;
        }
        // TOOLBUTTON
        // -------------------------------------------------------------------
        case CC_ToolButton: {

            const QToolButton *btn = (const QToolButton *) widget;
            
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
            
            int raised = ( flags & Style_Raised );
            int dx, dy;

            //QFontMetrics fm(*font);
            

            QToolBar* toolbar = 0;
            if(btn->parent() && btn->parent()->isWidgetType() && btn->parent()->inherits("QToolBar"))
                toolbar = static_cast<QToolBar*>(btn->parent());

            if(!highcolor || !toolbar)
                p->fillRect(x, y, w, h, cg.brush(QColorGroup::Background));
            else
            {
                // sync with drawKToolBar
                if(toolbar->orientation() == Qt::Horizontal)
                    // should be background fill?
                    drawVGradient(p, cg, QColorGroup::Background, x, y, w, h, x, y, toolbar->width()-2, toolbar->height()-2, true);
                else
                    drawHGradient(p, cg, QColorGroup::Background, x, y, w, h, x, y, toolbar->width()-2, toolbar->height()-2, true);
                p->setPen(cg.mid());
            p->drawLine(x, y+h-1, x+w, y+h-1);
            }

            if (on) down = true;
            if(raised || down) {
                int x2 = x+w;
                int y2 = y+h;
                p->setPen(cg.dark());
                p->drawLine(x+1, y+1, x2-2, y+1);
                p->drawLine(x, y+2, x, y2-3);
                p->drawLine(x2-1, y+2, x2-1, y2-3);
                p->drawLine(x+1, y2-2, x2-2, y2-2);

                p->setPen(down ? cg.mid() : cg.light());
                p->drawLine(x+1, y+2, x2-2, y+2);
                p->drawLine(x+1, y+2, x+1, y2-3);
                p->setPen(down ? cg.light() : cg.mid());
                p->drawLine(x2-2, y+3, x2-2, y2-3);
                p->drawLine(x+2, y2-3, x2-2, y2-3);
            }
            p->setPen(cg.text());

			break;
		}


		default:
			KStyle::drawComplexControl(control, p, widget,
						r, cg, flags, controls, active, opt);
			break;
	}
}


void QnxStyle::drawComplexControlMask( ComplexControl control,
											 QPainter *p,
											 const QWidget *widget,
											 const QRect &r,
											 const QStyleOption& opt ) const
{
	switch (control)
	{
		// COMBOBOX & TOOLBUTTON MASKS
		// -------------------------------------------------------------------
		case CC_ComboBox:
		case CC_ToolButton: {
			int x1, y1, x2, y2;
			r.coords( &x1, &y1, &x2, &y2 );
			QCOORD corners[] = { x1,y1, x2,y1, x1,y2, x2,y2 };
			p->fillRect( r, color1 );
			p->setPen( color0 );
			p->drawPoints( QPointArray(4, corners) );
			break;
		}

		default:
			KStyle::drawComplexControlMask(control, p, widget, r, opt);
	}
}


QRect QnxStyle::subRect(SubRect r, const QWidget *widget) const
{
	// We want the focus rect for buttons to be adjusted from
	// the Qt3 defaults to be similar to Qt 2's defaults.
	// -------------------------------------------------------------------
	if (r == SR_PushButtonFocusRect ) {
		const QPushButton* button = (const QPushButton*) widget;
		QRect wrect(widget->rect());
		int dbw1 = 0, dbw2 = 0;

		if (button->isDefault() || button->autoDefault()) {
			dbw1 = pixelMetric(PM_ButtonDefaultIndicator, widget);
			dbw2 = dbw1 * 2;
		}

		int dfw1 = pixelMetric(PM_DefaultFrameWidth, widget) * 2,
			dfw2 = dfw1 * 2;

		return QRect(wrect.x()      + dfw1 + dbw1 + 1,
					 wrect.y()      + dfw1 + dbw1 + 1,
					 wrect.width()  - dfw2 - dbw2 - 1,
					 wrect.height() - dfw2 - dbw2 - 1);
	} else
		return KStyle::subRect(r, widget);
}


int QnxStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
	switch(m)
	{
		// BUTTONS
		// -------------------------------------------------------------------
		case PM_ButtonMargin:				// Space btw. frame and label
			return 4;

		case PM_ButtonDefaultIndicator: {
				return 0;					// No indicator when highcolor
		}

		case PM_MenuButtonIndicator: {		// Arrow width
				return 7;
		}
										
		// CHECKBOXES / RADIO BUTTONS
		// -------------------------------------------------------------------
		case PM_ExclusiveIndicatorWidth:	// Radiobutton size
		case PM_ExclusiveIndicatorHeight:
		case PM_IndicatorWidth:				// Checkbox size
		case PM_IndicatorHeight: {
			return 13;						// 13x13
		}

		default:
			return KStyle::pixelMetric(m, widget);
	}
}


QSize QnxStyle::sizeFromContents( ContentsType contents,
										const QWidget* widget,
										const QSize &contentSize,
										const QStyleOption& opt ) const
{
	switch (contents)
	{
		// PUSHBUTTON SIZE
		// ------------------------------------------------------------------
		case CT_PushButton: {
			const QPushButton* button = (const QPushButton*) widget;
			int w  = contentSize.width();
			int h  = contentSize.height();
			int bm = pixelMetric( PM_ButtonMargin, widget );
			int fw = pixelMetric( PM_DefaultFrameWidth, widget ) * 2;

			w += bm + fw + 6;	// ### Add 6 to make way for bold font.
			h += bm + fw;

			// Ensure we stick to standard width and heights.
			if ( button->isDefault() || button->autoDefault() ) {
				if ( w < 80 && !button->pixmap() )
					w = 80;
			}

			if ( h < 22 )
				h = 22;

			return QSize( w, h );
		}


		// POPUPMENU ITEM SIZE
		// -----------------------------------------------------------------
		case CT_PopupMenuItem: {
			if ( ! widget || opt.isDefault() )
				return contentSize;

			const QPopupMenu *popup = (const QPopupMenu *) widget;
			bool checkable = popup->isCheckable();
			QMenuItem *mi = opt.menuItem();
			int maxpmw = opt.maxIconWidth();
			int w = contentSize.width(), h = contentSize.height();

			if ( mi->custom() ) {
				w = mi->custom()->sizeHint().width();
				h = mi->custom()->sizeHint().height();
				if ( ! mi->custom()->fullSpan() )
					h += 2*itemVMargin + 2*itemFrame;
			}
			else if ( mi->widget() ) {
			} else if ( mi->isSeparator() ) {
				w = 10; // Arbitrary
				h = 2;
			}
			else {
				if ( mi->pixmap() )
					h = QMAX( h, mi->pixmap()->height() + 2*itemFrame );
				else {
					// Ensure that the minimum height for text-only menu items
					// is the same as the icon size used by KDE.
					h = QMAX( h, 16 + 2*itemFrame );
					h = QMAX( h, popup->fontMetrics().height()
							+ 2*itemVMargin + 2*itemFrame );
				}
					
				if ( mi->iconSet() )
					h = QMAX( h, mi->iconSet()->pixmap(
								QIconSet::Small, QIconSet::Normal).height() +
								2 * itemFrame );
			}

			if ( ! mi->text().isNull() && mi->text().find('\t') >= 0 )
				w += 12;
			else if ( mi->popup() )
				w += 2 * arrowHMargin;

			if ( maxpmw )
				w += maxpmw + 6;
			if ( checkable && maxpmw < 20 )
				w += 20 - maxpmw;
			if ( checkable || maxpmw > 0 )
				w += 12;

			w += rightBorder;

			return QSize( w, h );
		}


		default:
			return KStyle::sizeFromContents( contents, widget, contentSize, opt );
	}
}

bool QnxStyle::eventFilter( QObject *obj, QEvent *ev )
{
    if(obj->inherits("KToolBar")){
        if(ev->type() == QEvent::Resize){
            QObjectList *tbChildList = obj->queryList("KToolBarButton", NULL,
                                                      false, false);
            QObjectListIt it(*tbChildList);
            QObject *child;
            while((child = it.current()) != NULL){
                ++it;
                if(child->isWidgetType())
                    ((QWidget *)child)->repaint(false);
            }
            delete tbChildList;
        }
    }
    else if(obj->inherits("QPushButton")){
        if(ev->type() == QEvent::Enter){
            QWidget *btn = (QWidget *)obj;
            if (btn->isEnabled()){
                //highlightWidget = btn;
                btn->repaint(false);
            }
        }
        else if(ev->type() == QEvent::Leave){
            QWidget *btn = (QWidget *)obj;
            //if (btn == highlightWidget)
            //  highlightWidget = 0L;
            btn->repaint(false);
        }
    }
    return(false);

}

void QnxStyle::drawVGradient(QPainter *p, const QColorGroup &g,
                            QColorGroup::ColorRole type, int x, int y,
                            int w, int h, int /*sx*/, int sy, int /*tw*/, int th, bool flip=false) const
{
    GradientSet *grSet = gDict.find(g.color(type).rgb());
    if(!grSet){
        grSet = new GradientSet(g.color(type));
        gDict.insert(g.color(type).rgb(), grSet);
    }

    int grheight = th - sy;
    if(grheight <= 16)
      {
          p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(VSmallestRev) : *grSet->gradient(VSmallest) );
      }
    else if(grheight <= 24){
         p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(VSmallRev) : *grSet->gradient(VSmall) );
    }
    else if(grheight <= 34){
           p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(VMedRev) : *grSet->gradient(VMed) );
    }
    else if(grheight <= 64){
          p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(VLargeRev) : *grSet->gradient(VLarge) );
    }
    else
      {
              KPixmap *vLarge =  ( flip ? grSet->gradient(VLargeRev) : grSet->gradient(VLarge) );
             p->fillRect(x, y+vLarge->height(), w, h-vLarge->height(), g.color(type).dark(110));
        p->drawTiledPixmap(x, y, w, vLarge->height(), *vLarge);
    }
}

void QnxStyle::drawHGradient(QPainter *p, const QColorGroup &g,
                            QColorGroup::ColorRole type, int x, int y,
                            int w, int h, int sx, int /*sy*/, int tw, int /*th*/, bool flip=false ) const
{
    GradientSet *grSet = gDict.find(g.color(type).rgb());
    if(!grSet)
      {
        grSet = new GradientSet(g.color(type));
        gDict.insert(g.color(type).rgb(), grSet);
    }
    int grwidth = tw - sx;
    if(grwidth <= 16)
      {
        p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(HSmallestRev) : *grSet->gradient(HSmallest) );
    }
    else if(grwidth <= 24){
        p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(HSmallRev) : *grSet->gradient(HSmallest) );
    }
    else if(grwidth <= 34)
      {
        p->drawTiledPixmap(x, y, w, h,  flip ? *grSet->gradient(HMedRev) : *grSet->gradient(HMed) );
    }
    else if(grwidth <= 52)
      {
        p->drawTiledPixmap(x, y, w, h, flip ? *grSet->gradient(HLarge) : *grSet->gradient(HLarge));
    }
    else
      {
        KPixmap *hLarge =  ( flip ? grSet->gradient(HLargeRev) : grSet->gradient(HLarge) );
        p->fillRect(x+hLarge->width(), y, w-hLarge->width(), h, g.background().dark(110));
        p->drawTiledPixmap(x, y, hLarge->width(), h, *hLarge);
    }
}
